
/* Authors: Patrick Derbez, Paul Huynh, Virginie Lallemand, 
 * Maria Naya-Plasencia, Leo Perrin, Andre Schrottenloher
 * Date: feb 2020
 */

#include <iostream>
#include <iomanip>
#include <cstdint>
#include <random>
#include <ctime>

#include <exception>

#include <array>
#include <tuple>
#include <vector>
#include "primitives.hpp"

class ClydeState {
public:
    static std::mt19937 PRNG;
    std::array<uint32_t, 4> x;

    ClydeState();
    
    ClydeState(uint32_t x0, uint32_t x1, uint32_t x2, uint32_t x3);
    
    ClydeState(const unsigned char* bytes);

    ClydeState(const ClydeState &c);

    ClydeState operator^(const ClydeState &c) const;

    bool operator==(const ClydeState &c) const;

    bool operator!=(const ClydeState &c) const;

    void print();
    
    int hw();
};


class ShadowState {
public:
    std::array<ClydeState, MLS_BUNDLES> c;

    ShadowState();

    ShadowState(const unsigned char* bytes);
    
    #if SMALL_PERM
    ShadowState(ClydeState c0, ClydeState c1, ClydeState c2);
    #else
    ShadowState(ClydeState c0, ClydeState c1, ClydeState c2, ClydeState c3);
    #endif

    ShadowState(const ShadowState &s);

    ShadowState operator^(const ShadowState &s) const;

    bool operator==(const ShadowState &s) const;

    bool operator!=(const ShadowState &s) const;

    void print();
    
    void to_uc(unsigned char result[]);
};



void print_stream(unsigned char * bytes);
ShadowState dbox_layer(ShadowState s);
ShadowState inverse_dbox_layer(ShadowState s);
ShadowState step_state(ShadowState start, unsigned int s);
ShadowState inverse_step_state(ShadowState start, unsigned int s);
ShadowState inverse_round_state(ShadowState start, unsigned int r);
unsigned int hw(unsigned char * bytes);



void test_shadow();


