/*
THIS FILE HAS BEEN MODIFIED FROM ITS ORIGINAL VERSION.
We have added a new parameter SHADOW_START
*/

/* Spook Reference Implementation v1
 *
 * Written in 2019 at UCLouvain (Belgium) by Olivier Bronchain, Gaetan Cassiers
 * and Charles Momin.
 * To the extent possible under law, the author(s) have dedicated all copyright
 * and related and neighboring rights to this software to the public domain
 * worldwide. This software is distributed without any warranty.
 *
 * You should have received a copy of the CC0 Public Domain Dedication along with
 * this software. If not, see <http://creativecommons.org/publicdomain/zero/1.0/>.
 */
#ifndef _H_PRIMITIVES_H_
#define _H_PRIMITIVES_H_

#include "parameters.h"
#include "utils.hpp"

#define CLYDE128_NBYTES 16

#if SMALL_PERM
#define SHADOW_NBYTES 48
#else
#define SHADOW_NBYTES 64
#endif // SMALL_PERM



#define CLYDE_128_NS 7                // Number of steps
#define CLYDE_128_NR 2 * CLYDE_128_NS // Number of rounds
#ifndef SHADOW_NS
#define SHADOW_NS 6                   // Number of steps
#endif
#ifndef SHADOW_START
#define SHADOW_START 0                // Starting step (included)
#endif

#ifndef SHADOW_NR
#define SHADOW_NR 2 * SHADOW_NS       // Number of rounds
#endif

#ifndef SHADOW_STARTR
#define SHADOW_STARTR 2 * SHADOW_START    // Starting round (included)
#endif

#define LS_ROWS 4      // Rows in the LS design
#define LS_ROW_BYTES 4 // number of bytes per row in the LS design
#define MLS_BUNDLES                                                     \
    (SHADOW_NBYTES / (LS_ROWS* LS_ROW_BYTES)) // Bundles in the mLS design


//extern "C" {

void clyde128_encrypt(unsigned char* c, const unsigned char* m,
                      const unsigned char* t, const unsigned char* k);

void clyde128_decrypt(unsigned char* m, const unsigned char* c,
                      const unsigned char* t, const unsigned char* k);

void shadow(unsigned char* x);

void sbox_layer(uint32_t* state);
void sbox_layer_inv(uint32_t* state);
void lbox(uint32_t* x, uint32_t* y);
void lbox_inv(uint32_t* x, uint32_t* y);
void lbox_layer(uint32_t* state);
void lbox_layer_inv(uint32_t* state);
void bytes2state(uint32_t* state, const unsigned char* byte);
void state2bytes(unsigned char* bytes, const uint32_t* state);
void xor_ls_state(uint32_t* state, const uint32_t* x);
void add_rc(uint32_t state[LS_ROWS], unsigned int round,
            unsigned int shift);
void dbox_mls_layer(uint32_t state[MLS_BUNDLES][LS_ROWS]);
void dbox_mls_layer_inv(uint32_t state[MLS_BUNDLES][LS_ROWS]);
//}


#endif //_H_PRIMITIVES_H_
